;************************************************************************
;*									*
;*				SPI LCD Driver				*
;*									*
;*			Author: P. Dannegger				*
;*				danni@specs.de				*
;*									*
;************************************************************************
;------------------------------------------------------------------------
;				Connection
;------------------------------------------------------------------------
;TINY12		74HCT164	LCD
;----------------------------------------
;LCD_CK		8
;LCD_E				E
;LCD_RS		1, 2		RS
;VCC		9, 14		VDD
;GND		7		VSS, VEE, R/W
;		3		D7
;		4		D6
;		5		D5
;		6		D4
;		10		D3
;		11		D2
;		12		D1
;		13		D0
;-------------------------------------------------------------------------
;				Initialize LCD
;-------------------------------------------------------------------------
;input: none
;output: none
;used: a0, a1, a2
;stack:	2
;
lcd_init:
	cbi	lcd_port, lcd_e
	cbi	lcd_port, lcd_ck		;shared with RTC
	sbi	lcd_ddr, lcd_e
	sbi	lcd_ddr, lcd_rs

	ldi	a0, 1 + xtal * 150 / 2560000	;wait 15ms
	rcall	_wait256
	ldi	a0, 0x38			;2 lines 5*7
	rcall	lcd_command

	ldi	a0, 1 + xtal * 41 / 2560000	;wait 4.1ms
	rcall	_wait256
;	ldi	a0, 0x38
	rcall	lcd_command

	ldi	a0, 1 + xtal * 1 / 2560000	;wait 100s
	rcall	_wait256
;	ldi	a0, 0x38
	rcall	lcd_command

	ldi	a0, 0x06
	rcall	lcd_command

	ldi	a0, 0x0C			;no cursor, no blink
	rcall	lcd_command
lcd_clear:
	ldi	a0, 0x01			;display clear
;-------------------------------------------------------------------------
;				Write Command to LCD
;-------------------------------------------------------------------------
;input: a0 = command
;output: a0 = unchanged
;used: a1, a2
;stack:	1
;
lcd_command:
	clc
	rjmp	_lcd_out
;-------------------------------------------------------------------------
;				Write Data to LCD
;-------------------------------------------------------------------------
;input: a0 = data
;output: a0 = unchanged
;used: a1, a2
;stack:	1
;
lcd_dot:				;display decimal point
	ldi	a0, '.'
lcd_data:
	inc	lcd_count			;count digits
	sec
;-------------------------------------------------------------------------
;internal subroutine
;
_lcd_out:
	ldi	a1, 9
_lou1:	sbi     lcd_ddr, lcd_ck			;clk = 0
	cbi     lcd_ddr, lcd_ck			;clk = 1
	ror	a0
	cbi	lcd_port, lcd_rs
	brcc	_lou2
	sbi	lcd_port, lcd_rs
_lou2:	dec	a1
	brne	_lou1
	sbi	lcd_port, lcd_e
	cbi	lcd_port, lcd_e
	brcs	_wai2
	cpi	a0, 4
	brcc	_wai2
	ldi	a1, 1 + xtal * 16 / 2560000	;1.6ms
;-------------------------------------------------------------------------
;internal subroutine
;
wait_256c:
_wait256:					;Tw = 256 cycle * a0
	clr	a2
_wai1:	subi	a2, 4
	sbci	a1, 0
	brne	_wai1
_wai2:	ret
;-------------------------------------------------------------------------
